@@ -44,7 +44,8 @@ const struct vpn_config invalid_cfg = {
 	.gateway_host = {'\0'},
 	.gateway_port = 0,
 	.username = {'\0'},
-	.password = NULL,
+	.password = {'\0'},
+	.password_set = 0,
 	.otp = {'\0'},
 	.otp_prompt = NULL,
 	.otp_delay = -1,
@@ -74,6 +75,8 @@ const struct vpn_config invalid_cfg = {
 	.ca_file = NULL,
 	.user_cert = NULL,
 	.user_key = NULL,
+	.pem_passphrase = {'\0'},
+	.pem_passphrase_set = 0,
 	.insecure_ssl = -1,
 	.cipher_list = NULL,
 	.min_tls = -1,
@@ -221,7 +224,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 		// Expect something like: "key = value"
 		equals = strchr(line, '=');
 		if (equals == NULL) {
-			log_warn("Bad line in config file: \"%s\".\n", line);
+			log_warn("Bad line in configuration file: \"%s\".\n", line);
 			continue;
 		}
 		equals[0] = '\0';
@@ -248,25 +251,27 @@ int load_config(struct vpn_config *cfg, const char *filename)
 		}
 
 		if (strcmp(key, "host") == 0) {
-			strncpy(cfg->gateway_host, val, FIELD_SIZE);
-			cfg->gateway_host[FIELD_SIZE] = '\0';
+			strncpy(cfg->gateway_host, val, GATEWAY_HOST_SIZE);
+			cfg->gateway_host[GATEWAY_HOST_SIZE] = '\0';
 		} else if (strcmp(key, "port") == 0) {
 			unsigned long port = strtoul(val, NULL, 0);
 
 			if (port == 0 || port > 65535) {
-				log_warn("Bad port in config file: \"%lu\".\n",
+				log_warn("Bad port in configuration file: \"%lu\".\n",
 				         port);
 				continue;
 			}
 			cfg->gateway_port = port;
 		} else if (strcmp(key, "username") == 0) {
-			strncpy(cfg->username, val, FIELD_SIZE - 1);
-			cfg->username[FIELD_SIZE] = '\0';
+			strncpy(cfg->username, val, USERNAME_SIZE);
+			cfg->username[USERNAME_SIZE] = '\0';
 		} else if (strcmp(key, "password") == 0) {
-			cfg->password = strdup(val);
+			strncpy(cfg->password, val, PASSWORD_SIZE);
+			cfg->password[PASSWORD_SIZE] = '\0';
+			cfg->password_set = 1;
 		} else if (strcmp(key, "otp") == 0) {
-			strncpy(cfg->otp, val, FIELD_SIZE - 1);
-			cfg->otp[FIELD_SIZE] = '\0';
+			strncpy(cfg->otp, val, OTP_SIZE);
+			cfg->otp[OTP_SIZE] = '\0';
 		} else if (strcmp(key, "otp-prompt") == 0) {
 			free(cfg->otp_prompt);
 			cfg->otp_prompt = strdup(val);
@@ -274,7 +279,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			long otp_delay = strtol(val, NULL, 0);
 
 			if (otp_delay < 0 || otp_delay > UINT_MAX) {
-				log_warn("Bad value for otp-delay in config file: \"%s\".\n",
+				log_warn("Bad value for otp-delay in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -283,7 +288,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int no_ftm_push = strtob(val);
 
 			if (no_ftm_push < 0) {
-				log_warn("Bad no-ftm-push in config file: \"%s\".\n",
+				log_warn("Bad no-ftm-push in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -292,13 +297,13 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			free(cfg->pinentry);
 			cfg->pinentry = strdup(val);
 		} else if (strcmp(key, "realm") == 0) {
-			strncpy(cfg->realm, val, FIELD_SIZE - 1);
-			cfg->realm[FIELD_SIZE] = '\0';
+			strncpy(cfg->realm, val, REALM_SIZE);
+			cfg->realm[REALM_SIZE] = '\0';
 		} else if (strcmp(key, "set-dns") == 0) {
 			int set_dns = strtob(val);
 
 			if (set_dns < 0) {
-				log_warn("Bad set-dns in config file: \"%s\".\n",
+				log_warn("Bad set-dns in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -307,7 +312,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int set_routes = strtob(val);
 
 			if (set_routes < 0) {
-				log_warn("Bad set-routes in config file: \"%s\".\n",
+				log_warn("Bad set-routes in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -316,7 +321,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int half_internet_routes = strtob(val);
 
 			if (half_internet_routes < 0) {
-				log_warn("Bad half-internet-routes in config file: \"%s\".\n",
+				log_warn("Bad half-internet-routes in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -325,7 +330,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			unsigned long persistent = strtoul(val, NULL, 0);
 
 			if (persistent > UINT_MAX) {
-				log_warn("Bad value for persistent in config file: \"%s\".\n",
+				log_warn("Bad value for persistent in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -335,7 +340,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int pppd_use_peerdns = strtob(val);
 
 			if (pppd_use_peerdns < 0) {
-				log_warn("Bad pppd-use-peerdns in config file: \"%s\".\n",
+				log_warn("Bad pppd-use-peerdns in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -370,7 +375,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int use_resolvconf = strtob(val);
 
 			if (use_resolvconf < 0) {
-				log_warn("Bad use-resolvconf value in config file: \"%s\".\n",
+				log_warn("Bad use-resolvconf value in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -382,14 +387,14 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int use_syslog = strtob(val);
 
 			if (use_syslog < 0) {
-				log_warn("Bad use-syslog in config file: \"%s\".\n",
+				log_warn("Bad use-syslog in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
 			cfg->use_syslog = use_syslog;
 		} else if (strcmp(key, "trusted-cert") == 0) {
 			if (strlen(val) != SHA256STRLEN - 1) {
-				log_warn("Bad certificate sha256 digest in config file: \"%s\".\n",
+				log_warn("Bad certificate sha256 digest in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -407,11 +412,15 @@ int load_config(struct vpn_config *cfg, const char *filename)
 		} else if (strcmp(key, "user-key") == 0) {
 			free(cfg->user_key);
 			cfg->user_key = strdup(val);
+		} else if (strcmp(key, "pem-passphrase") == 0) {
+			strncpy(cfg->pem_passphrase, val, PEM_PASSPHRASE_SIZE);
+			cfg->pem_passphrase[PEM_PASSPHRASE_SIZE] = '\0';
+			cfg->pem_passphrase_set = 1;
 		} else if (strcmp(key, "insecure-ssl") == 0) {
 			int insecure_ssl = strtob(val);
 
 			if (insecure_ssl < 0) {
-				log_warn("Bad insecure-ssl in config file: \"%s\".\n",
+				log_warn("Bad insecure-ssl in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -424,7 +433,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int min_tls = parse_min_tls(val);
 
 			if (min_tls == -1) {
-				log_warn("Bad min-tls in config file: \"%s\".\n",
+				log_warn("Bad min-tls in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			} else {
@@ -435,7 +444,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int seclevel_1 = strtob(val);
 
 			if (seclevel_1 < 0) {
-				log_warn("Bad seclevel-1 in config file: \"%s\".\n",
+				log_warn("Bad seclevel-1 in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
@@ -450,7 +459,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			free(cfg->check_virtual_desktop);
 			cfg->check_virtual_desktop = strdup(val);
 		} else {
-			log_warn("Bad key in config file: \"%s\".\n", key);
+			log_warn("Bad key in configuration file: \"%s\".\n", key);
 			goto err_free;
 		}
 	}
@@ -468,7 +477,6 @@ err_close:
 
 void destroy_vpn_config(struct vpn_config *cfg)
 {
-	free(cfg->password);
 	free(cfg->otp_prompt);
 	free(cfg->pinentry);
 #if HAVE_USR_SBIN_PPPD
@@ -485,6 +493,9 @@ void destroy_vpn_config(struct vpn_config *cfg)
 	free(cfg->user_cert);
 	free(cfg->user_key);
 	free(cfg->cipher_list);
+	free(cfg->user_agent);
+	free(cfg->hostcheck);
+	free(cfg->check_virtual_desktop);
 	while (cfg->cert_whitelist != NULL) {
 		struct x509_digest *tmp = cfg->cert_whitelist->next;
 
@@ -501,8 +512,10 @@ void merge_config(struct vpn_config *dst, struct vpn_config *src)
 		dst->gateway_port = src->gateway_port;
 	if (src->username[0])
 		strcpy(dst->username, src->username);
-	if (src->password != NULL && src->password[0])
-		dst->password = strdup(src->password);
+	if (src->password_set) {
+		strcpy(dst->password, src->password);
+		dst->password_set = src->password_set;
+	}
 	if (src->otp[0])
 		strcpy(dst->otp, src->otp);
 	if (src->otp_delay != invalid_cfg.otp_delay)
@@ -575,6 +588,10 @@ void merge_config(struct vpn_config *dst, struct vpn_config *src)
 		free(dst->user_key);
 		dst->user_key = src->user_key;
 	}
+	if (src->pem_passphrase_set) {
+		strcpy(dst->pem_passphrase, src->pem_passphrase);
+		dst->pem_passphrase_set = src->pem_passphrase_set;
+	}
 	if (src->insecure_ssl != invalid_cfg.insecure_ssl)
 		dst->insecure_ssl = src->insecure_ssl;
 	if (src->cipher_list) {
