@@ -19,6 +19,7 @@
 #define OPENFORTIVPN_CONFIG_H
 
 #include <netinet/in.h>
+#include <net/if.h>
 
 #include <errno.h>
 #include <stdint.h>
@@ -57,7 +58,12 @@ struct x509_digest {
 	char data[SHA256STRLEN];
 };
 
-#define FIELD_SIZE	64
+#define GATEWAY_HOST_SIZE	253
+#define USERNAME_SIZE	64
+#define PASSWORD_SIZE	256
+#define OTP_SIZE	64
+#define REALM_SIZE	63
+#define PEM_PASSPHRASE_SIZE	31
 
 /*
  * RFC 6265 does not limit the size of cookies:
@@ -77,18 +83,19 @@ struct x509_digest {
 #define MAX_DOMAIN_LENGTH 256
 
 struct vpn_config {
-	char		gateway_host[FIELD_SIZE + 1];
+	char		gateway_host[GATEWAY_HOST_SIZE + 1];
 	struct in_addr	gateway_ip;
 	uint16_t	gateway_port;
-	char		username[FIELD_SIZE + 1];
-	char		*password;
-	char		otp[FIELD_SIZE + 1];
+	char		username[USERNAME_SIZE + 1];
+	char		password[PASSWORD_SIZE + 1];
+	int		password_set;
+	char		otp[OTP_SIZE + 1];
 	char		*otp_prompt;
-	unsigned int  otp_delay;
-	int         no_ftm_push;
+	unsigned int	otp_delay;
+	int		no_ftm_push;
 	char		*pinentry;
-	char		iface_name[FIELD_SIZE + 1];
-	char		realm[FIELD_SIZE + 1];
+	char		iface_name[IF_NAMESIZE];
+	char		realm[REALM_SIZE + 1];
 
 	int	set_routes;
 	int	set_dns;
@@ -114,15 +121,17 @@ struct vpn_config {
 	char			*ca_file;
 	char			*user_cert;
 	char			*user_key;
+	char			pem_passphrase[PEM_PASSPHRASE_SIZE + 1];
+	int			pem_passphrase_set;
 	int			insecure_ssl;
 	int			min_tls;
 	int			seclevel_1;
 	char			*cipher_list;
 	struct x509_digest	*cert_whitelist;
-	int                     use_engine;
-	char                    *user_agent;
-	char                    *hostcheck;
-	char                    *check_virtual_desktop;
+	int			use_engine;
+	char			*user_agent;
+	char			*hostcheck;
+	char			*check_virtual_desktop;
 };
 
 int add_trusted_cert(struct vpn_config *cfg, const char *digest);
