@@ -87,7 +87,7 @@ int http_send(struct tunnel *tunnel, const char *request, ...)
 	strcpy(logbuffer, buffer);
 	if (loglevel <= OFV_LOG_DEBUG_DETAILS && tunnel->config->password[0] != '\0') {
 		char *pwstart;
-		char password[3 * FIELD_SIZE + 1];
+		char password[3 * PASSWORD_SIZE + 1];
 
 		url_encode(password, tunnel->config->password);
 		pwstart = strstr(logbuffer, password);
@@ -197,7 +197,7 @@ int http_receive(struct tunnel *tunnel,
 				                                 header_size);
 
 				if (header)
-					content_size = atoi(header);
+					content_size = strtol(header, NULL, 10);
 
 				if (find_header(buffer,
 				                "Transfer-Encoding: chunked",
@@ -515,7 +515,7 @@ static int try_otp_auth(struct tunnel *tunnel, const char *buffer,
 		}
 	}
 	if (p == NULL)
-		p = "Please enter one-time password:";
+		p = "Please enter one-time password: ";
 	/* Search for all inputs */
 	while ((s = strcasestr(s, "<INPUT"))) {
 		s += 6;
@@ -583,8 +583,13 @@ static int try_otp_auth(struct tunnel *tunnel, const char *buffer,
 
 			v = NULL;
 			if (cfg->otp[0] == '\0') {
-				read_password(cfg->pinentry, "otp",
-				              p, cfg->otp, FIELD_SIZE);
+				// Interactively ask user for OTP
+				char hint[USERNAME_SIZE + 1 + REALM_SIZE + 1 + GATEWAY_HOST_SIZE + 5];
+
+				sprintf(hint, "%s_%s_%s_otp",
+				        cfg->username, cfg->realm, cfg->gateway_host);
+				read_password(cfg->pinentry, hint,
+				              p, cfg->otp, OTP_SIZE);
 				if (cfg->otp[0] == '\0') {
 					log_error("No OTP specified\n");
 					return 0;
@@ -627,16 +632,17 @@ static int try_otp_auth(struct tunnel *tunnel, const char *buffer,
 int auth_log_in(struct tunnel *tunnel)
 {
 	int ret;
-	char username[3 * FIELD_SIZE + 1];
-	char password[3 * FIELD_SIZE + 1];
-	char realm[3 * FIELD_SIZE + 1];
+	char username[3 * USERNAME_SIZE + 1];
+	char password[3 * PASSWORD_SIZE + 1];
+	char realm[3 * REALM_SIZE + 1];
 	char reqid[32] = { '\0' };
 	char polid[32] = { '\0' };
 	char group[128] = { '\0' };
 	char portal[64] = { '\0' };
 	char magic[32] = {'\0' };
 	char peer[32]  = { '\0' };
-	char data[1152], token[128], tokenresponse[256], tokenparams[320];
+	char data[9 + 3 * USERNAME_SIZE + 12 + 3 * PASSWORD_SIZE + 7 + 3 * REALM_SIZE + 7 + 1];
+	char token[128], tokenresponse[256], tokenparams[320];
 	char action_url[1024] = { '\0' };
 	char *res = NULL;
 	uint32_t response_size;
@@ -726,10 +732,14 @@ int auth_log_in(struct tunnel *tunnel)
 			snprintf(tokenparams, sizeof(tokenparams), "ftmpush=1");
 		} else {
 			if (cfg->otp[0] == '\0') {
-				// Prompt for 2FA token
-				read_password(cfg->pinentry, "2fa",
+				// Interactively ask user for 2FA token
+				char hint[USERNAME_SIZE + 1 + REALM_SIZE + 1 + GATEWAY_HOST_SIZE + 5];
+
+				sprintf(hint, "%s_%s_%s_2fa",
+				        cfg->username, cfg->realm, cfg->gateway_host);
+				read_password(cfg->pinentry, hint,
 				              "Two-factor authentication token: ",
-				              cfg->otp, FIELD_SIZE);
+				              cfg->otp, OTP_SIZE);
 
 				if (cfg->otp[0] == '\0') {
 					log_error("No token specified\n");
