@@ -15,13 +15,15 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _OPENFORTIVPN_IO_H
-#define _OPENFORTIVPN_IO_H
+#ifndef OPENFORTIVPN_IO_H
+#define OPENFORTIVPN_IO_H
 
-#include <stdint.h>
 #include <sys/types.h>
 #include <pthread.h>
 
+#include <stddef.h>
+#include <stdint.h>
+
 /*
  * For performance reasons, we store the 6-byte header used by the SSL
  * communication right in front of the real PPP packet data. This way,
