@@ -30,8 +30,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define PWD_BUFSIZ	4096
-
 #if HAVE_USR_SBIN_PPPD && HAVE_USR_SBIN_PPP
 #error "Both HAVE_USR_SBIN_PPPD and HAVE_USR_SBIN_PPP have been defined."
 #elif HAVE_USR_SBIN_PPPD
@@ -110,7 +108,7 @@ PPPD_USAGE \
 "Options:\n" \
 "  -h --help                     Show this help message and exit.\n" \
 "  --version                     Show version and exit.\n" \
-"  -c <file>, --config=<file>    Specify a custom config file (default:\n" \
+"  -c <file>, --config=<file>    Specify a custom configuration file (default:\n" \
 "                                " SYSCONFDIR "/openfortivpn/config).\n" \
 "  -u <user>, --username=<user>  VPN account username.\n" \
 "  -p <pass>, --password=<pass>  VPN account password.\n" \
@@ -138,6 +136,7 @@ PPPD_USAGE \
 "  --user-cert=pkcs11:           Use smartcard. Takes also partial or full PKCS11-URI.\n" \
 "  --user-key=<file>             Use specified PEM-encoded key if the server requires\n" \
 "                                authentication with a certificate.\n" \
+"  --pem-passphrase=<pass>       Pass phrase for the PEM-encoded key.\n" \
 "  --use-syslog                  Log to syslog instead of terminal.\n" \
 "  --trusted-cert=<digest>       Trust a given gateway. If classical SSL\n" \
 "                                certificate validation fails, the gateway\n" \
@@ -161,7 +160,7 @@ PPPD_USAGE \
 "                                (compiled in) list of ciphers. This lowers limits on\n" \
 "                                dh key." help_seclevel_1 "\n" \
 "  --persistent=<interval>       Run the vpn persistently in a loop and try to re-\n" \
-"                                connect every <interval> seconds when dropping out\n" \
+"                                connect every <interval> seconds when dropping out.\n" \
 "  -v                            Increase verbosity. Can be used multiple times\n" \
 "                                to be even more verbose.\n" \
 "  -q                            Decrease verbosity. Can be used multiple times\n" \
@@ -169,12 +168,12 @@ PPPD_USAGE \
 
 #define help_config \
 "\n" \
-"Config file:\n" \
+"Configuration file:\n" \
 "  Options can be taken from a configuration file. Options passed in the\n" \
-"  command line will override those from the config file, though. The default\n" \
-"  config file is " SYSCONFDIR "/openfortivpn/config,\n" \
+"  command line will override those from the configuration file, though. The\n" \
+"  default configuration file is " SYSCONFDIR "/openfortivpn/config,\n" \
 "  but this can be set using the -c option.\n" \
-"  A simple config file example looks like:\n" \
+"  A simple configuration file example looks like:\n" \
 "      # this is a comment\n" \
 "      host = vpn-gateway\n" \
 "      port = 8443\n" \
@@ -182,7 +181,7 @@ PPPD_USAGE \
 "      password = bar\n" \
 "      trusted-cert = certificatedigest4daa8c5fe6c...\n" \
 "      trusted-cert = othercertificatedigest6631bf...\n" \
-"  For a full-featured config see man openfortivpn(1).\n"
+"  For a full-featured configuration see man openfortivpn(1).\n"
 
 int main(int argc, char **argv)
 {
@@ -195,7 +194,8 @@ int main(int argc, char **argv)
 		.gateway_host = {'\0'},
 		.gateway_port = 443,
 		.username = {'\0'},
-		.password = NULL,
+		.password = {'\0'},
+		.password_set = 0,
 		.otp = {'\0'},
 		.otp_prompt = NULL,
 		.otp_delay = 0,
@@ -225,6 +225,8 @@ int main(int argc, char **argv)
 		.ca_file = NULL,
 		.user_cert = NULL,
 		.user_key = NULL,
+		.pem_passphrase = {'\0'},
+		.pem_passphrase_set = 0,
 		.insecure_ssl = 0,
 #ifdef TLS1_2_VERSION
 		.min_tls = TLS1_2_VERSION,
@@ -235,47 +237,48 @@ int main(int argc, char **argv)
 		.cipher_list = NULL,
 		.cert_whitelist = NULL,
 		.use_engine = 0,
-		.user_agent = "Mozilla/5.0 SV1",
+		.user_agent = NULL,
 	};
 	struct vpn_config cli_cfg = invalid_cfg;
 
 	const struct option long_options[] = {
-		{"help",            no_argument,       NULL, 'h'},
-		{"version",         no_argument,       NULL, 0},
-		{"config",          required_argument, NULL, 'c'},
-		{"pinentry",        required_argument, NULL, 0},
-		{"realm",           required_argument, NULL, 0},
-		{"username",        required_argument, NULL, 'u'},
-		{"password",        required_argument, NULL, 'p'},
-		{"otp",             required_argument, NULL, 'o'},
-		{"otp-prompt",      required_argument, NULL, 0},
-		{"otp-delay",       required_argument, NULL, 0},
-		{"no-ftm-push",     no_argument, &cli_cfg.no_ftm_push, 1},
-		{"ifname",          required_argument, NULL, 0},
-		{"set-routes",	    required_argument, NULL, 0},
-		{"no-routes",       no_argument, &cli_cfg.set_routes, 0},
+		{"help",                 no_argument,       NULL, 'h'},
+		{"version",              no_argument,       NULL, 0},
+		{"config",               required_argument, NULL, 'c'},
+		{"pinentry",             required_argument, NULL, 0},
+		{"realm",                required_argument, NULL, 0},
+		{"username",             required_argument, NULL, 'u'},
+		{"password",             required_argument, NULL, 'p'},
+		{"otp",                  required_argument, NULL, 'o'},
+		{"otp-prompt",           required_argument, NULL, 0},
+		{"otp-delay",            required_argument, NULL, 0},
+		{"no-ftm-push",          no_argument, &cli_cfg.no_ftm_push, 1},
+		{"ifname",               required_argument, NULL, 0},
+		{"set-routes",	         required_argument, NULL, 0},
+		{"no-routes",            no_argument, &cli_cfg.set_routes, 0},
 		{"half-internet-routes", required_argument, NULL, 0},
-		{"set-dns",         required_argument, NULL, 0},
-		{"no-dns",          no_argument, &cli_cfg.set_dns, 0},
-		{"use-syslog",      no_argument, &cli_cfg.use_syslog, 1},
-		{"persistent",      required_argument, NULL, 0},
-		{"ca-file",         required_argument, NULL, 0},
-		{"user-cert",       required_argument, NULL, 0},
-		{"user-key",        required_argument, NULL, 0},
-		{"trusted-cert",    required_argument, NULL, 0},
-		{"insecure-ssl",    no_argument, &cli_cfg.insecure_ssl, 1},
-		{"cipher-list",     required_argument, NULL, 0},
-		{"min-tls",         required_argument, NULL, 0},
-		{"seclevel-1",      no_argument, &cli_cfg.seclevel_1, 1},
+		{"set-dns",              required_argument, NULL, 0},
+		{"no-dns",               no_argument, &cli_cfg.set_dns, 0},
+		{"use-syslog",           no_argument, &cli_cfg.use_syslog, 1},
+		{"persistent",           required_argument, NULL, 0},
+		{"ca-file",              required_argument, NULL, 0},
+		{"user-cert",            required_argument, NULL, 0},
+		{"user-key",             required_argument, NULL, 0},
+		{"pem-passphrase",       required_argument, NULL, 0},
+		{"trusted-cert",         required_argument, NULL, 0},
+		{"insecure-ssl",         no_argument, &cli_cfg.insecure_ssl, 1},
+		{"cipher-list",          required_argument, NULL, 0},
+		{"min-tls",              required_argument, NULL, 0},
+		{"seclevel-1",           no_argument, &cli_cfg.seclevel_1, 1},
 #if HAVE_USR_SBIN_PPPD
-		{"pppd-use-peerdns", required_argument, NULL, 0},
-		{"pppd-no-peerdns", no_argument, &cli_cfg.pppd_use_peerdns, 0},
-		{"pppd-log",        required_argument, NULL, 0},
-		{"pppd-plugin",     required_argument, NULL, 0},
-		{"pppd-ipparam",    required_argument, NULL, 0},
-		{"pppd-ifname",     required_argument, NULL, 0},
-		{"pppd-call",       required_argument, NULL, 0},
-		{"plugin",          required_argument, NULL, 0}, // deprecated
+		{"pppd-use-peerdns",     required_argument, NULL, 0},
+		{"pppd-no-peerdns",      no_argument, &cli_cfg.pppd_use_peerdns, 0},
+		{"pppd-log",             required_argument, NULL, 0},
+		{"pppd-plugin",          required_argument, NULL, 0},
+		{"pppd-ipparam",         required_argument, NULL, 0},
+		{"pppd-ifname",          required_argument, NULL, 0},
+		{"pppd-call",            required_argument, NULL, 0},
+		{"plugin",               required_argument, NULL, 0}, // deprecated
 #endif
 #if HAVE_USR_SBIN_PPP
 		{"ppp-system",      required_argument, NULL, 0},
@@ -394,6 +397,14 @@ int main(int argc, char **argv)
 				cli_cfg.user_key = strdup(optarg);
 				break;
 			}
+			if (strcmp(long_options[option_index].name,
+			           "pem-passphrase") == 0) {
+				strncpy(cli_cfg.pem_passphrase, optarg,
+				        PEM_PASSPHRASE_SIZE);
+				cli_cfg.pem_passphrase[PEM_PASSPHRASE_SIZE] = '\0';
+				cli_cfg.pem_passphrase_set = 1;
+				break;
+			}
 			if (strcmp(long_options[option_index].name,
 			           "pinentry") == 0) {
 				cli_cfg.pinentry = strdup(optarg);
@@ -401,8 +412,8 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "realm") == 0) {
-				strncpy(cli_cfg.realm, optarg, FIELD_SIZE);
-				cli_cfg.realm[FIELD_SIZE] = '\0';
+				strncpy(cli_cfg.realm, optarg, REALM_SIZE);
+				cli_cfg.realm[REALM_SIZE] = '\0';
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
@@ -435,8 +446,8 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "ifname") == 0) {
-				strncpy(cli_cfg.iface_name, optarg, FIELD_SIZE);
-				cli_cfg.iface_name[FIELD_SIZE] = '\0';
+				strncpy(cli_cfg.iface_name, optarg, IF_NAMESIZE - 1);
+				cli_cfg.iface_name[IF_NAMESIZE - 1] = '\0';
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
@@ -515,17 +526,19 @@ int main(int argc, char **argv)
 			config_file = optarg;
 			break;
 		case 'u':
-			strncpy(cli_cfg.username, optarg, FIELD_SIZE);
-			cli_cfg.username[FIELD_SIZE] = '\0';
+			strncpy(cli_cfg.username, optarg, USERNAME_SIZE);
+			cli_cfg.username[USERNAME_SIZE] = '\0';
 			break;
 		case 'p':
-			cli_cfg.password = strdup(optarg);
+			strncpy(cli_cfg.password, optarg, PASSWORD_SIZE);
+			cli_cfg.password[PASSWORD_SIZE] = '\0';
+			cli_cfg.password_set = 1;
 			while (*optarg)
 				*optarg++ = '*';  // nuke it
 			break;
 		case 'o':
-			strncpy(cli_cfg.otp, optarg, FIELD_SIZE);
-			cli_cfg.otp[FIELD_SIZE] = '\0';
+			strncpy(cli_cfg.otp, optarg, OTP_SIZE);
+			cli_cfg.otp[OTP_SIZE] = '\0';
 			break;
 		default:
 			goto user_error;
@@ -535,8 +548,8 @@ int main(int argc, char **argv)
 	if (optind < argc - 1 || optind > argc)
 		goto user_error;
 
-	if (cli_cfg.password != NULL && cli_cfg.password[0] != '\0')
-		log_warn("You should not pass the password on the command line. Type it interactively or use a config file instead.\n");
+	if (cli_cfg.password[0] != '\0')
+		log_warn("You should not pass the password on the command line. Type it interactively or use a configuration file instead.\n");
 
 	log_debug_all("ATTENTION: the output contains sensitive information such as the THE CLEAR TEXT PASSWORD.\n");
 
@@ -544,27 +557,35 @@ int main(int argc, char **argv)
 	if (strcmp(&REVISION[1], VERSION))
 		log_debug("revision " REVISION "\n");
 
-	// Load config file
+	// Load configuration file
 	if (config_file[0] != '\0') {
 		ret = load_config(&cfg, config_file);
 		if (ret == 0)
-			log_debug("Loaded config file \"%s\".\n", config_file);
+			log_debug("Loaded configuration file \"%s\".\n", config_file);
 		else
-			log_warn("Could not load config file \"%s\" (%s).\n",
+			log_warn("Could not load configuration file \"%s\" (%s).\n",
 			         config_file, err_cfg_str(ret));
 	}
-	if (cfg.password != NULL && cli_cfg.password == NULL) {
+	if (cli_cfg.password_set) {
+		if (cli_cfg.password[0] == '\0')
+			log_debug("Disabled password due to empty command-line option\n");
+	} else if (cfg.password_set) {
 		if (cfg.password[0] == '\0')
-			log_debug("Disabled password due to empty entry in config file \"%s\"\n",
+			log_debug("Disabled password due to empty entry in configuration file \"%s\"\n",
 			          config_file);
 		else
-			log_debug("Loaded password from config file \"%s\"\n",
+			log_debug("Loaded password from configuration file \"%s\"\n",
 			          config_file);
 	}
-	// Then apply CLI config
+
+	// Then apply CLI configuration
 	merge_config(&cfg, &cli_cfg);
 	set_syslog(cfg.use_syslog);
 
+	// Set default UA
+	if (cfg.user_agent == NULL)
+		cfg.user_agent = strdup("Mozilla/5.0 SV1");
+
 	// Read host and port from the command line
 	if (optind == argc - 1) {
 		host = argv[optind++];
@@ -578,8 +599,8 @@ int main(int argc, char **argv)
 				goto user_error;
 			}
 		}
-		strncpy(cfg.gateway_host, host, FIELD_SIZE);
-		cfg.gateway_host[FIELD_SIZE] = '\0';
+		strncpy(cfg.gateway_host, host, GATEWAY_HOST_SIZE);
+		cfg.gateway_host[GATEWAY_HOST_SIZE] = '\0';
 	}
 
 	// Check host and port
@@ -588,33 +609,28 @@ int main(int argc, char **argv)
 		goto user_error;
 	}
 	// Check username
-	if (cfg.username[0] == '\0' && cfg.user_cert == NULL) {
-		log_error("Specify a username.\n");
-		goto user_error;
-	}
-	// If username but no password given, interactively ask user
-	if (cfg.password == NULL && cfg.username[0] != '\0') {
-		char *tmp_password = malloc(PWD_BUFSIZ); // allocate large buffer
-
-		if (tmp_password == NULL) {
-			log_error("malloc: %s\n", strerror(errno));
-			ret = EXIT_FAILURE;
-			goto exit;
+	if (cfg.username[0] == '\0')
+		// Need either username or cert
+		if (cfg.user_cert == NULL) {
+			log_error("Specify a username.\n");
+			goto user_error;
 		}
-		read_password(cfg.pinentry, "password",
-		              "VPN account password: ", tmp_password, PWD_BUFSIZ);
-		cfg.password = strdup(tmp_password); // copy string of correct size
-		free(tmp_password);
+	// If username but no password given, interactively ask user
+	if (!cfg.password_set && cfg.username[0] != '\0') {
+		char hint[USERNAME_SIZE + 1 + REALM_SIZE + 1 + GATEWAY_HOST_SIZE + 10];
+
+		sprintf(hint, "%s_%s_%s_password",
+		        cfg.username, cfg.realm, cfg.gateway_host);
+		read_password(cfg.pinentry, hint,
+		              "VPN account password: ",
+		              cfg.password, PASSWORD_SIZE);
 	}
-	log_debug("Config host = \"%s\"\n", cfg.gateway_host);
-	log_debug("Config realm = \"%s\"\n", cfg.realm);
-	log_debug("Config port = \"%d\"\n", cfg.gateway_port);
+	log_debug("Configuration host = \"%s\"\n", cfg.gateway_host);
+	log_debug("Configuration realm = \"%s\"\n", cfg.realm);
+	log_debug("Configuration port = \"%d\"\n", cfg.gateway_port);
 	if (cfg.username[0] != '\0')
-		log_debug("Config username = \"%s\"\n", cfg.username);
-	if (cfg.password != NULL)
-		log_debug_all("Config password = \"%s\"\n", cfg.password);
-	else
-		cfg.password = strdup("");
+		log_debug("Configuration username = \"%s\"\n", cfg.username);
+	log_debug_all("Configuration password = \"%s\"\n", cfg.password);
 	if (cfg.otp[0] != '\0')
 		log_debug("One-time password = \"%s\"\n", cfg.otp);
 
