@@ -37,10 +37,9 @@ static char *uri_escape(const char *string)
 	char *escaped = NULL;
 	int allocated_len = 0;
 	int real_len = 0;
-	int i;
 
-	for (i = 0; string[i]; i++) {
-		if (allocated_len + 4 >= real_len) {
+	while (*string != '\0') {
+		if (allocated_len < real_len + 4) {
 			allocated_len += 16;
 			char *tmp = realloc(escaped, allocated_len);
 
@@ -52,10 +51,13 @@ static char *uri_escape(const char *string)
 			}
 			escaped = tmp;
 		}
-		if (isalnum(string[i]))
-			escaped[real_len++] = string[i];
+		if (isalnum(*string) || *string == '-' || *string == '_' ||
+		    *string == '.' || *string == '~')
+			escaped[real_len++] = *string;
 		else
-			real_len += sprintf(&escaped[real_len], "%%%02X", string[i]);
+			real_len += sprintf(&escaped[real_len], "%%%02X",
+			                    (unsigned char)*string);
+		string++;
 	}
 	if (escaped)
 		escaped[real_len] = '\0';
@@ -143,7 +145,7 @@ static int pinentry_read(int from, char **retstr)
 	}
 
 	if (strncmp(buf, "ERR ", 4) == 0 || strncmp(buf, "S ERROR", 7) == 0) {
-		ret = atoi(&buf[4]);
+		ret = strtol(&buf[4], NULL, 10);
 		if (!ret)
 			ret = -1;
 		if (retstr) {
@@ -213,20 +215,20 @@ static void pinentry_read_password(const char *pinentry, const char *hint,
 		close(to_pinentry[1]);
 		if (dup2(to_pinentry[0], STDIN_FILENO) == -1) {
 			perror("dup2");
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 		close(to_pinentry[0]);
 
 		close(from_pinentry[0]);
 		if (dup2(from_pinentry[1], STDOUT_FILENO) == -1) {
 			perror("dup2");
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 		close(from_pinentry[1]);
 
 		execlp(pinentry, pinentry, NULL);
 		perror(pinentry);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	close(to_pinentry[0]);
